//
// Created by Иван_Архипов on 24.11.2017.
//

#include "Subfiles/OggSubfile.h"

#include "BinaryData.h"
#include "DatFile.h"
#include "DatException.h"
#include "SubfileData.h"

namespace LOTRO_DAT {
    OggSubfile::OggSubfile() = default;

    OggSubfile::OggSubfile(DatFile *dat, long long dictionary_offset, long long fragments_count, long long unknown1,
                           long long file_id, long long file_offset, long long file_size,
                           long long timestamp,
                           long long version, long long block_size)
            : Subfile(dat, dictionary_offset, fragments_count, unknown1, file_id, file_offset, file_size, timestamp, version, block_size) {
    }

    FILE_TYPE OggSubfile::FileType() const {
        return OGG;
    }

    std::string OggSubfile::Extension() const {
        return ".ogg";
    }

    SubfileData OggSubfile::PrepareForExport(const BinaryData &file_data) {
        SubfileData result;
        result.binary_data = file_data.CutData(8);
        result.options["fid"] = file_id();
        result.options["ext"] = Extension();
        return result;
    }

    BinaryData OggSubfile::MakeForImport(const BinaryData &old_data, const SubfileData &data) {
        if (!data.options["ext"] || data.options["ext"].as<std::string>() != Extension() ||
            !data.options["fid"] || data.options["fid"].as<long long>() != file_id()) {
            throw DatException("Bad DdsSubfile::MakeForImport() - invalid options data!", IMPORT_EXCEPTION);
        }
        BinaryData file_size = BinaryData::FromNumber<4>(data.binary_data.size() - 8);
        BinaryData file_id = BinaryData::FromNumber<4>(this->file_id());

        return old_data.CutData(0, 8) + file_id + file_size + data.binary_data;
    }
};